@@ -3,6 +3,10 @@
     if(missing(x)) stop("Argument x should be specified")
     if(!(class(x) %in% c("hlme","lcmm","Jointlcmm","multlcmm"))) stop("Use with hlme, lcmm, multlcmm or Jointlcmm only")
     if(missing(var.time)) stop("Argument var.time should be specified")
+    if(class(x)=="lcmm")
+        {
+            if(x$linktype==3) stop("This function is not available for thresholds mixed models")
+        }
     
     data <- eval(x$call$data)
 
