@@ -113,6 +113,10 @@ summary.lcmm <- function(object,...)
                 wald <- NA
                 pwald <- NA
                 coef <- x$best
+
+                sech <- rep(NA,length(coef))
+                waldch <- rep(NA,length(coef))
+                pwaldch <- rep(NA,length(coef))
             }
 
         ##prendre abs pour les parametres mis au carre
@@ -120,10 +124,17 @@ summary.lcmm <- function(object,...)
         if(ncor>0) coef[NPROB+NEF+NVC+NW+ntrtot+ncor] <- abs(coef[NPROB+NEF+NVC+NW+ntrtot+ncor])
 
         ## convertir en character
-        coefch <- format(as.numeric(sprintf("%.5f",coef)),nsmall=5,scientific=FALSE)
-        sech <- format(as.numeric(sprintf("%.5f",se)),nsmall=5,scientific=FALSE)
-        waldch <- format(as.numeric(sprintf("%.3f",wald)),nsmall=3,scientific=FALSE)
-        pwaldch <- format(as.numeric(sprintf("%.5f",pwald)),nsmall=5,scientific=FALSE)
+        if(x$conv!=2)
+            {
+                coefch <- format(as.numeric(sprintf("%.5f",coef)),nsmall=5,scientific=FALSE)
+                sech <- format(as.numeric(sprintf("%.5f",se)),nsmall=5,scientific=FALSE)
+                waldch <- format(as.numeric(sprintf("%.3f",wald)),nsmall=3,scientific=FALSE)
+                pwaldch <- format(as.numeric(sprintf("%.5f",pwald)),nsmall=5,scientific=FALSE)
+            }
+        else
+            {
+                coefch <- format(as.numeric(sprintf("%.5f",coef)),nsmall=5,scientific=FALSE)
+            }
 
         if(length(posfix))
             {
